/*
 * Copyright 2009 Cedric Priscal
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package android.serialport.sample;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.security.InvalidParameterException;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.DialogInterface.OnClickListener;
import android.os.Bundle;
import android.serialport.SerialPort;
import android.util.Log;

public abstract class SerialPortActivity extends Activity {

	protected Application mApplication;
	protected SerialPort mSerialPort;
	protected OutputStream mOutputStream;
	private InputStream mInputStream;
	private ReadThread mReadThread;

	private class ReadThread extends Thread {

		private static final String TAG = "SerialTest";

		@Override
		public void run() {
			super.run();
			while(!isInterrupted()) {
				int size;
				try {
					byte[] buffer = new byte[64];
					String str ="";
					
					if (mInputStream == null) return;
					size = mInputStream.read(buffer);
					for (int i = 0; i < size; i++)
						 str += (char)buffer[i];					
					Log.d(TAG, "ReadString : "+str);
					if(str.equals("OK+RENEW"))
					{
						mOutputStream.write(new String("AT+RESET").getBytes());
					}
					if(str.equals("OK+RESET"))
					{
						mOutputStream.write(new String("AT").getBytes());
					}
					if(str.equals("OK"))
					{
						
						mOutputStream.write(new String("AT+MARJ0x1234").getBytes());
					}		

					if(str.equals("OK+Set:0x1234"))
					{
						mOutputStream.write(new String("AT+MINO0xFA01").getBytes());
					}	
					if(str.equals("OK+Set:0xFA01"))
					{
						mOutputStream.write(new String("AT+ADVI5").getBytes());
					}		
					if(str.equals("OK+Set:5"))
					{
						mOutputStream.write(new String("AT+NAMEDOPEY").getBytes());
					}	
					if(str.equals("OK+Set:DOPEY"))
					{
						//mOutputStream.write(new String("AT+ADTY3").getBytes());
						//mOutputStream.write(new String("AT+VERR?").getBytes());
						mOutputStream.write(new String("AT+IBEA1").getBytes());
					}	
					if(str.equals("OK+Set:3"))
					{
						Log.d(TAG, "in AT+IBEA1");
						//mOutputStream.write(new String("AT+IBEA?").getBytes());
						//mOutputStream.write(new String("AT+HELP?").getBytes());
						mOutputStream.write(new String("AT+VERR?").getBytes());
					}
					if(str.equals("OK+Set:1"))
					{
						Log.d(TAG, "in AT+BUSHU");
						mOutputStream.write(new String("AT+BUSHU2").getBytes());
					}	
					if(str.equals("OK+BUSHU:2"))
					{
						mOutputStream.write(new String("AT+PWRM0 ").getBytes());
					}
					if(str.equals("OK+Set:0"))
					{
						mOutputStream.write(new String("AT+RESET ").getBytes());
					}						
					if (size > 0) {
						onDataReceived(buffer, size);
					}
				} catch (IOException e) {
					e.printStackTrace();
					return;
				}
			}
		}
	}

	private void DisplayError(int resourceId) {
		AlertDialog.Builder b = new AlertDialog.Builder(this);
		b.setTitle("Error");
		b.setMessage(resourceId);
		b.setPositiveButton("OK", new OnClickListener() {
			public void onClick(DialogInterface dialog, int which) {
				SerialPortActivity.this.finish();
			}
		});
		b.show();
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		mApplication = (Application) getApplication();
		try {
			mSerialPort = mApplication.getSerialPort();
			mOutputStream = mSerialPort.getOutputStream();
			mInputStream = mSerialPort.getInputStream();
			//mOutputStream.write(new String("AT+RESET").getBytes());
			//mOutputStream.write(0x0D);
			/* Create a receiving thread */
			mReadThread = new ReadThread();
			mReadThread.start();
		} catch (SecurityException e) {
			DisplayError(R.string.error_security);
		} catch (IOException e) {
			DisplayError(R.string.error_unknown);
		} catch (InvalidParameterException e) {
			DisplayError(R.string.error_configuration);
		}
	}

	protected abstract void onDataReceived(final byte[] buffer, final int size);

	@Override
	protected void onDestroy() {
		if (mReadThread != null)
			mReadThread.interrupt();
		mApplication.closeSerialPort();
		mSerialPort = null;
		super.onDestroy();
	}
}
